@@ -29,7 +29,7 @@ from .float_data import FloatData
 from .geometric_data_constants import GeometricDataConstants
 from .integer_data import IntegerData
 from .primitive_type_enum import PrimitiveTypeEnum
-from .reference_value_map import ReferenceValue, ReferenceValueMap
+from .reference_value_map import ReferenceValueMap
 from .referenced_data import ReferencedData
 from .text_data import CommentsData, TextData
 from .unknown_data import UnknownData
