@@ -16,17 +16,28 @@
 #  along with geoh5py.  If not, see <https://www.gnu.org/licenses/>.
 
 from .data_type import DataType
-from .float_data import FloatData
+from .integer_data import IntegerData
 from .primitive_type_enum import PrimitiveTypeEnum
-from .reference_value_map import ReferenceValueMap
 
 
-class ReferencedData(FloatData):
+class ReferencedData(IntegerData):
+    """
+    Reference data described by indices and associated strings.
+    """
+
     def __init__(self, data_type: DataType, **kwargs):
         super().__init__(data_type, **kwargs)
 
-        self._value_map = ReferenceValueMap()
+        if "value_map" in kwargs.keys():
+            self.entity_type.value_map = kwargs["value_map"]
 
     @classmethod
     def primitive_type(cls) -> PrimitiveTypeEnum:
         return PrimitiveTypeEnum.REFERENCED
+
+    @property
+    def value_map(self):
+        """
+        Pointer to the :obj:`data.data_type.DataType.value_map`
+        """
+        return self.entity_type.value_map
